@@ -70,6 +70,7 @@ def embed(
     default: Any = None,
     dimensions: int | None = None,
     resource_group: str | None = None,
+    venv: str | None = None,
     resource_hint: ResourceHint | None = None,
 ) -> Any:
     """Specify an embedding feature.
@@ -181,6 +182,9 @@ def embed(
         The resource group for the embed resolver: this is used to isolate execution of
         the resolver onto a separate pod (or set of nodes), allowing model inference
         to be run in a separate environment, such as on a GPU-enabled node.
+    venv
+        A virtual environment to use for the resolver. This is used to isolate the resolver
+        from the default requirements, allowing different versions of packages to be used.
     resource_hint
         The resource hint for the embed resolver: Can be either CPU, GPU, or IO.
         Chalk uses the resource hint to optimize resolver execution.
@@ -269,6 +273,7 @@ def embed(
                         parse=None,
                         resource_hint=resource_hint,
                         resource_group=resource_group,
+                        venv=venv,
                         static=False,
                         total=False,
                         autogenerated=False,
@@ -400,6 +405,7 @@ def embed(
                 parse=None,
                 resource_hint=resource_hint,
                 resource_group=resource_group,
+                venv=venv,
                 static=False,
                 total=False,
                 autogenerated=False,
