@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import typing
 from typing import Protocol, Sequence, Type
 
 import pyarrow as pa
@@ -28,6 +29,12 @@ class EmbeddingProvider(Protocol):
         additional columns are for metadata (such as the prompt, with instructor)"""
         ...
 
+    def async_generate_embedding(self, input: pa.Table) -> typing.AsyncGenerator[pa.FixedSizeListArray, None]:
+        """Generate embeddings from the input table. The input table will contain all string columns.
+        The embedding implementation should have a well defined order for which column index is for the the feature that will be embedded, and which
+        additional columns are for metadata (such as the prompt, with instructor)"""
+        ...
+
     def get_vector_class(self) -> Type[Vector]:
         """Get the expected output class, with the vector dimensions and dtype"""
         ...
