@@ -8,6 +8,7 @@ import json
 import operator
 import os
 import pathlib
+import typing
 import warnings
 from datetime import date, datetime, timedelta, timezone
 from typing import (
@@ -63,18 +64,13 @@ if TYPE_CHECKING:
     from pydantic import BaseModel
 
     from chalk.features.feature_set import Features
-else:
-    try:
-        from pydantic.v1 import BaseModel
-    except ImportError:
-        from pydantic import BaseModel
 
 TIntStr = TypeVar("TIntStr", int, str)
+DataFrameGetOptions = TypeVar("DataFrameGetOptions", bool, Filter, Underscore, FeatureWrapper, Feature, str)
 
 
 class DataFrameMeta(type):
     def __getitem__(cls, item: Any) -> Type[DataFrame]:
-
         from chalk.features.feature_set import Features
 
         # leaving untyped as we type the individual features as their object type
@@ -213,15 +209,15 @@ class DataFrameMeta(type):
 
     @property
     def references_feature_set(cls):
-        from chalk.features.feature_set import FeatureSetBase
-
         cls = cast(Type[DataFrame], cls)
         if cls.__references_feature_set__ is not None:
             return cls.__references_feature_set__
         else:
             # Determine the unique @features cls that encompasses all columns
             root_ns = get_unique_item((x.root_namespace for x in cls.__columns__), "root ns")
-        return FeatureSetBase.registry[root_ns]
+        from chalk.features.feature_set import CURRENT_FEATURE_REGISTRY
+
+        return CURRENT_FEATURE_REGISTRY.get().get_feature_sets()[root_ns]
 
     @property
     def namespace(cls) -> str:
@@ -231,7 +227,7 @@ class DataFrameMeta(type):
         namespaces = (x for x in namespaces if not x.startswith("__chalk__"))
         return get_unique_item(namespaces, f"dataframe {cls.__name__} column namespaces")
 
-    def _parse_underscore(self, exp: Underscore, pydantic_model: Optional[BaseModel] = None) -> Any:
+    def _parse_underscore(self, exp: Underscore, pydantic_model: "Optional[BaseModel]" = None) -> Any:
         if pydantic_model:
             context = self
             is_pydantic = True
@@ -242,12 +238,19 @@ class DataFrameMeta(type):
 
 
 class DataFrame(metaclass=DataFrameMeta):
+    """Chalk's DataFrame class models tabular data similar to how `pandas` and `polars` do.
+    DataFrame can be used a Chalk feature type, when defining has-many joins between feature
+    classes. For more about the  Chalk DataFrame, see: https://docs.chalk.ai/docs/dataframe
+
+    NOTE: DataFrame functions are meant to be used in Python resolvers or in notebooks, but
+    NOT in Chalk Expressions."""
+
     filters: ClassVar[Tuple[Filter, ...]] = ()
     columns: Tuple[Feature, ...]  # set via a @property on the metaclass
     __columns__: ClassVar[Tuple[Feature, ...]] = ()
     references_feature_set: Optional[Type[Features]]  # set via a @property on the metaclass
     __references_feature_set__: ClassVar[Optional[Type[Features]]] = None
-    __pydantic_model__: ClassVar[Optional[Type[BaseModel]]] = None
+    __pydantic_model__: "ClassVar[Optional[Type[BaseModel]]]" = None
     __limit__: int | None = None
     """The maximum number of rows to return"""
 
@@ -276,7 +279,7 @@ class DataFrame(metaclass=DataFrameMeta):
         # in which case the data types should no longer be converted.
         # This is an undocumented parameter, so it does not appear in the docstring
         convert_dtypes: bool = True,
-        pydantic_model: Optional[Type[BaseModel]] = None,
+        pydantic_model: "Optional[Type[BaseModel]]" = None,
         verify_validity: bool = True,
     ):
         """Construct a Chalk `DataFrame`.
@@ -335,7 +338,7 @@ class DataFrame(metaclass=DataFrameMeta):
         ... }))
         """
         super().__init__()
-        from chalk.features.feature_set import Features, FeatureSetBase
+        from chalk.features.feature_set import CURRENT_FEATURE_REGISTRY, Features
 
         try:
             import polars as pl
@@ -393,6 +396,11 @@ class DataFrame(metaclass=DataFrameMeta):
 
                     if features_typ != type(row):
                         raise ValueError("Cannot mix different feature classes in a DataFrame")
+
+                    # If there are multiple versions of a feature, then the `features` may
+                    # include the same value more than once. In this case, we want to only
+                    # push one of them into the result.
+                    already_placed_fqns: set[str] = set()
                     for x in row.features:
                         assert isinstance(x, Feature)
                         assert x.attribute_name is not None
@@ -408,6 +416,9 @@ class DataFrame(metaclass=DataFrameMeta):
                             raise ValueError(
                                 f"While constructing a dataframe from a list of feature classes, feature '{x.root_fqn}' was defined in some rows but not in others. Features must be defined in all provided rows or none of the provided rows."
                             )
+                        if x.root_fqn in already_placed_fqns:
+                            continue
+                        already_placed_fqns.add(x.root_fqn)
                         new_data[x.root_fqn].append(feature_val)
                 data = new_data
             if isinstance(data, dict):
@@ -480,7 +491,8 @@ class DataFrame(metaclass=DataFrameMeta):
         namespaces_set = {x.root_namespace for x in self.columns if not x.root_namespace == "__chalk__"}
         if len(namespaces_set) == 1:
             self.namespace = get_unique_item(namespaces_set, "DataFrame column namespaces")
-            self.references_feature_set = FeatureSetBase.registry[self.namespace]
+            feature_sets = CURRENT_FEATURE_REGISTRY.get().get_feature_sets()
+            self.references_feature_set = feature_sets[self.namespace]
         else:
             # Allow empty dataframes or dataframes with multiple namespaces
             self.namespace = None
@@ -503,6 +515,14 @@ class DataFrame(metaclass=DataFrameMeta):
     #######################
     # Filtering / Selecting
     #######################
+    @typing.overload
+    def __getitem__(self, item: int) -> Features:
+        ...
+
+    @typing.overload
+    def __getitem__(self, item: Union[DataFrameGetOptions, Sequence[DataFrameGetOptions]]) -> DataFrame:
+        ...
+
     def __getitem__(self, item: Any) -> Union[Features, DataFrame]:
         """Filter the rows of a `DataFrame` or project out columns.
 
@@ -566,7 +586,10 @@ class DataFrame(metaclass=DataFrameMeta):
             )
             return list(df.to_features())[0]
 
-        has_bool_or_filter_value = any(isinstance(x, (bool, Filter, Underscore)) for x in ensure_tuple(item))
+        has_bool_or_filter_value = any(
+            isinstance(x, (bool, Filter, Underscore))  # pyright: ignore[reportUnnecessaryIsInstance]
+            for x in ensure_tuple(item)
+        )
         if has_bool_or_filter_value:
             # If we have a boolean or Filter value, then that means we need to ast-parse the caller since
             # python has already evaluated AND, OR, and IN operations into literal booleans or Filters
@@ -577,7 +600,14 @@ class DataFrame(metaclass=DataFrameMeta):
             item = parse_dataframe_getitem()
 
         # Resolve underscores to be within the dataframe's context
-        item = tuple(self._parse_underscore(exp=x) if isinstance(x, Underscore) else x for x in ensure_tuple(item))
+        item = tuple(
+            (
+                self._parse_underscore(exp=x)
+                if isinstance(x, Underscore)  # pyright: ignore[reportUnnecessaryIsInstance]
+                else x
+            )
+            for x in ensure_tuple(item)
+        )
         assert all(not isinstance(x, Underscore) for x in item)
         if len(item) == 1:
             item = item[0]
@@ -609,7 +639,8 @@ class DataFrame(metaclass=DataFrameMeta):
         group: Mapping[Union[Feature, Any], Union[Feature, Any]],
         agg: Mapping[Any, Any],
     ) -> DataFrame:
-        """Aggregate the `DataFrame` by the specified columns.
+        """Aggregate the `DataFrame` by the specified columns. This can be used
+        in Python resolvers or notebooks.
 
         Parameters
         ----------
@@ -647,13 +678,17 @@ class DataFrame(metaclass=DataFrameMeta):
         """
         import polars as pl
 
-        from chalk.features.feature_set import FeatureSetBase
+        from chalk.features.feature_set import CURRENT_FEATURE_REGISTRY
 
         groupby = [pl.col(str(v)).alias(str(k)) for k, v in group.items()]
 
         namespaces = {unwrap_feature(a).namespace for a in agg.keys()}
         namespace = namespaces.pop() if len(namespaces) == 1 else None
-        timestamp_feature = None if namespace is None else FeatureSetBase.registry[namespace].__chalk_ts__
+        if namespace is not None:
+            features_cls = CURRENT_FEATURE_REGISTRY.get().get_feature_sets()[namespace]
+            timestamp_feature = features_cls.__chalk_ts__
+        else:
+            timestamp_feature = None
 
         now = datetime.now(tz=timezone.utc)
         cols = []
@@ -698,7 +733,8 @@ class DataFrame(metaclass=DataFrameMeta):
         column: Any | None = None,
         descending: bool = False,
     ) -> list[int]:
-        """Compute a histogram with fixed width bins.
+        """Compute a histogram with fixed width bins. This can be used
+        in Python resolvers or notebooks.
 
         Parameters
         ----------
@@ -751,7 +787,7 @@ class DataFrame(metaclass=DataFrameMeta):
             bin_width = bin_width.total_seconds() * 1_000_000.0
 
         if isinstance(base, (date, datetime)):
-            epoch = datetime.utcfromtimestamp(0)
+            epoch = datetime.fromtimestamp(0, timezone.utc)
             if isinstance(base, datetime):
                 if base.tzinfo is not None:
                     epoch = epoch.replace(tzinfo=base.tzinfo)
@@ -839,6 +875,8 @@ class DataFrame(metaclass=DataFrameMeta):
         In the above example, the sixth time bucket is empty, and
         will not be included in the resulting `DataFrame`.
 
+        This can be used in Python resolvers or notebooks.
+
         Parameters
         ----------
         index
@@ -906,13 +944,18 @@ class DataFrame(metaclass=DataFrameMeta):
         """
         import polars as pl
 
-        from chalk.features.feature_set import FeatureSetBase
+        from chalk.features.feature_set import CURRENT_FEATURE_REGISTRY
 
         groupby = [pl.col(str(v)).alias(str(k)) for k, v in group.items()] if group is not None else None
 
         namespaces = {unwrap_feature(a).namespace for a in agg.keys()}
         namespace = namespaces.pop() if len(namespaces) == 1 else None
-        timestamp_feature = None if namespace is None else FeatureSetBase.registry[namespace].__chalk_ts__
+
+        if namespace is not None:
+            features_cls = CURRENT_FEATURE_REGISTRY.get().get_feature_sets()[namespace]
+            timestamp_feature = features_cls.__chalk_ts__
+        else:
+            timestamp_feature = None
 
         now = get_filter_now()
         cols: List[pl.Expr] = []
@@ -966,6 +1009,7 @@ class DataFrame(metaclass=DataFrameMeta):
         """Vertically stack the `DataFrame` with another `DataFrame`
         containing the same columns. The `DataFrame` other will
         be appended to the bottom of this `DataFrame`.
+        This can be used in Python resolvers or notebooks.
 
         Parameters
         ----------
@@ -994,6 +1038,7 @@ class DataFrame(metaclass=DataFrameMeta):
 
     def num_unique(self, column: Any = None) -> int:
         """Return the number of unique values in the specified column.
+        This can be used in Python resolvers or notebooks.
 
         Parameters
         ----------
@@ -1117,7 +1162,7 @@ class DataFrame(metaclass=DataFrameMeta):
         """
         import polars as pl
 
-        from chalk.features.feature_set import FeatureSetBase
+        from chalk.features.feature_set import CURRENT_FEATURE_REGISTRY
 
         new_c = {}
         for k, v in c.items():
@@ -1136,7 +1181,11 @@ class DataFrame(metaclass=DataFrameMeta):
 
         namespaces = {unwrap_feature(a).namespace for a in c.keys()}
         namespace = namespaces.pop() if len(namespaces) == 1 else None
-        timestamp_feature = None if namespace is None else FeatureSetBase.registry[namespace].__chalk_ts__
+        if namespace is not None:
+            features_cls = CURRENT_FEATURE_REGISTRY.get().get_feature_sets()[namespace]
+            timestamp_feature = features_cls.__chalk_ts__
+        else:
+            timestamp_feature = None
 
         now = get_filter_now()
         cols: List[pl.Expr] = []
@@ -1375,11 +1424,12 @@ class DataFrame(metaclass=DataFrameMeta):
         else:
             cols_to_select, dtypes, new_columns = cls._parse_columns(columns)
 
+        # 'dtypes' deprecated for 'schema_overrides' in polars 0.20+, but parameter renamed without breaking
         data = pl.read_csv(
             source=path,
             has_header=has_header,
             columns=cols_to_select,
-            dtypes=dtypes,
+            dtypes=dtypes,  # pyright: ignore[reportCallIssue]
             new_columns=new_columns,
             storage_options=DataFrame._get_storage_options(),
         )
@@ -1660,7 +1710,8 @@ class DataFrame(metaclass=DataFrameMeta):
             return True
 
         if any(
-            (not isinstance(x, type) or not issubclass(x, pl.Boolean)) and x != pl.Boolean
+            (not isinstance(x, type) or not issubclass(x, pl.Boolean))
+            and x != pl.Boolean  # pyright: ignore[reportUnnecessaryComparison]
             for x in self._underlying.dtypes
         ):
             raise TypeError("DataFrame.any() is not defined on a DataFrame that contains non-boolean columns.")
@@ -2094,7 +2145,7 @@ class DataFrame(metaclass=DataFrameMeta):
             SpaceShip(id=2, volume=5000)
         ]
         """
-        from chalk.features.feature_set import FeatureSetBase
+        from chalk.features.feature_set import CURRENT_FEATURE_REGISTRY
 
         if self.namespace is None and self._pydantic_model is None:
             raise ValueError("This method is not supported if the DataFrame spans multiple namespaces")
@@ -2138,7 +2189,7 @@ class DataFrame(metaclass=DataFrameMeta):
                 if rooted_prefix == self.namespace:
                     assert not values_are_lists, "Top level feature shouldn't be a has-many"
                     assert self.namespace is not None
-                    features_cls = FeatureSetBase.registry[self.namespace]
+                    features_cls = CURRENT_FEATURE_REGISTRY.get().get_feature_sets()[self.namespace]
                     ans.append(features_cls(**sub_kwargs))
                 else:
                     rooted_prefix_split = rooted_prefix.split(".")
