@@ -21,7 +21,7 @@ from chalk._gen.chalk.graph.v1 import graph_pb2
 from chalk._gen.chalk.graph.v1 import graph_pb2 as pb
 from chalk._gen.chalk.graph.v1.graph_pb2 import CronFilterWithFeatureArgs, SourceFileReference
 from chalk._gen.chalk.graph.v2 import sources_pb2 as sources_pb
-from chalk._gen.chalk.lsp.v1.lsp_pb2 import Position, Range
+from chalk._gen.chalk.lsp.v1.lsp_pb2 import Location, Position, Range
 from chalk._validation.feature_validation import FeatureValidation
 from chalk.features import (
     CacheStrategy,
@@ -126,6 +126,21 @@ class ToProtoConverter:
         else:
             return convert_value_to_proto_expr(v)
 
+    @classmethod
+    def convert_expression_definition_location(cls, v: Union[Underscore, TPrimitive]) -> Location | None:
+        if isinstance(v, Underscore):
+            if (dl := v.definition_location()) is None:
+                return None
+            # TODO: Dominic - see if we can maintain more specific line range info
+            return Location(
+                uri=dl.file,
+                range=Range(
+                    start=Position(line=dl.line, character=dl.column), end=Position(line=dl.line, character=dl.column)
+                ),
+            )
+        else:
+            return None
+
     @staticmethod
     def _convert_stream_source(source: StreamSource) -> sources_pb.StreamSource:
         options: Dict[str, TJSON] = source.config_to_dict()
@@ -975,6 +990,11 @@ class ToProtoConverter:
                 expression=ToProtoConverter.convert_underscore(f.underscore_expression)
                 if f.underscore_expression is not None
                 else None,
+                expression_definition_location=ToProtoConverter.convert_expression_definition_location(
+                    f.underscore_expression
+                )
+                if f.underscore_expression is not None
+                else None,
                 no_display=f.no_display,
                 attribute_name=f.attribute_name if hasattr(f, "attribute_name") else None,
                 unversioned_attribute_name=f.unversioned_attribute_name
@@ -1133,6 +1153,7 @@ class ToProtoConverter:
             static_operation=static_operation,
             sql_settings=ToProtoConverter.convert_sql_settings(r.sql_settings) if r.sql_settings else None,
             output_row_order=r.output_row_order,
+            venv=r.venv,
         )
 
     @staticmethod
